CREATE SEQUENCE empleado_seq 
    START WITH 1 
    INCREMENT BY 1 
    MAXVALUE 9999 
    MINVALUE 1 
    NOCACHE;

CREATE SEQUENCE servicio_seq 
    START WITH 1 
    INCREMENT BY 1 
    MAXVALUE 9999 
    MINVALUE 1 
    NOCACHE;

CREATE SEQUENCE turno_seq 
    START WITH 1 
    INCREMENT BY 1 
    MAXVALUE 9999 
    MINVALUE 1 
    NOCACHE;

CREATE SEQUENCE solicitud_turno_seq 
    START WITH 1 
    INCREMENT BY 1 
    MAXVALUE 9999 
    MINVALUE 1 
    NOCACHE;


-- TABLA SERVICIO

CREATE TABLE servicio (
    id_servicio       NUMBER(4)      NOT NULL,
    nombre_servicio   VARCHAR2(20)   NOT NULL
)
LOGGING;

ALTER TABLE servicio 
    ADD CONSTRAINT servicio_pk 
    PRIMARY KEY (id_servicio);

ALTER TABLE servicio 
    ADD CONSTRAINT servicio_un 
    UNIQUE (nombre_servicio);

CREATE OR REPLACE TRIGGER trg_servicio_ai
BEFORE INSERT ON servicio
FOR EACH ROW
BEGIN
    IF :NEW.id_servicio IS NULL THEN
        SELECT servicio_seq.NEXTVAL
          INTO :NEW.id_servicio
          FROM DUAL;
    END IF;
END;
/

-- TABLA EMPLEADO

CREATE TABLE empleado (
    id_empleado   NUMBER(4)      NOT NULL,
    nombre        VARCHAR2(20)   NOT NULL,
    apellido1     VARCHAR2(20)   NOT NULL,
    apellido2     VARCHAR2(20),
    contrasena    CHAR(52)       NOT NULL,
    dni           CHAR(9)        NOT NULL,
    email         VARCHAR2(52)   NOT NULL,
    alias         VARCHAR2(20)   NOT NULL,
    tipo_empleado CHAR(1)        NOT NULL,
    id_servicio   NUMBER(4)      NOT NULL
)
LOGGING;

ALTER TABLE empleado
    ADD CONSTRAINT email_ck 
    CHECK (email LIKE '%_@__%.__%');

ALTER TABLE empleado
    ADD CONSTRAINT tipo_empleado_ck 
    CHECK (tipo_empleado IN ('A','E'));

ALTER TABLE empleado
    ADD CONSTRAINT empleado_pk 
    PRIMARY KEY (id_empleado);

ALTER TABLE empleado
    ADD CONSTRAINT empleado_dni_un 
    UNIQUE (dni);

ALTER TABLE empleado
    ADD CONSTRAINT empleado_email_un 
    UNIQUE (email);

ALTER TABLE empleado
    ADD CONSTRAINT empleado_alias_un 
    UNIQUE (alias);

ALTER TABLE empleado
    ADD CONSTRAINT empleado_servicio_fk 
    FOREIGN KEY (id_servicio)
    REFERENCES servicio (id_servicio)
    NOT DEFERRABLE;

CREATE OR REPLACE TRIGGER trg_empleado_ai
BEFORE INSERT ON empleado
FOR EACH ROW
BEGIN
    IF :NEW.id_empleado IS NULL THEN
        SELECT empleado_seq.NEXTVAL
          INTO :NEW.id_empleado
          FROM DUAL;
    END IF;
END;
/

-- TABLA TURNO

CREATE TABLE turno (
    id_turno      NUMBER(4)      NOT NULL,
    tipo_turno    CHAR(1)        NOT NULL,
    fecha_turno   DATE           DEFAULT SYSDATE NOT NULL,
    id_empleado   NUMBER(4)      NOT NULL
)
LOGGING;

ALTER TABLE turno
    ADD CONSTRAINT turno_pk 
    PRIMARY KEY (id_turno);

ALTER TABLE turno 
    ADD CONSTRAINT turno_tipo_ck 
    CHECK (tipo_turno IN ('M','T','N'));

ALTER TABLE turno
    ADD CONSTRAINT turno_empleado_fk
    FOREIGN KEY (id_empleado)
    REFERENCES empleado (id_empleado)
    NOT DEFERRABLE;

CREATE OR REPLACE TRIGGER trg_turno_ai
BEFORE INSERT ON turno
FOR EACH ROW
BEGIN
    IF :NEW.id_turno IS NULL THEN
        SELECT turno_seq.NEXTVAL
          INTO :NEW.id_turno
          FROM DUAL;
    END IF;
END;
/

-- TABLA SOLICITUD_TURNO

CREATE TABLE solicitud_turno (
    id_solicitud_cambio  NUMBER(4)      NOT NULL,
    id_turno_origen      TIMESTAMP      NOT NULL,
    id_turno_destino     TIMESTAMP      NOT NULL,
    estado_solicitud     CHAR(1)        NOT NULL,
    id_turno             NUMBER(4)      NOT NULL
)
LOGGING;

ALTER TABLE solicitud_turno
    ADD CONSTRAINT solicitud_turno_pk 
    PRIMARY KEY (id_solicitud);
   
ALTER TABLE solicitud_turno 
    ADD CONSTRAINT estado_solicitud_ck
    CHECK (estado_solicitud IN ('P','A'));
    
ALTER TABLE solicitud_turno
    ADD CONSTRAINT turno_solicitud_fk
    FOREIGN KEY (id_turno)
    REFERENCES turno (id_turno)
    NOT DEFERRABLE;

CREATE OR REPLACE TRIGGER trg_solicitud_turno_ai
BEFORE INSERT ON solicitud_turno
FOR EACH ROW
BEGIN
    IF :NEW.id_solicitud IS NULL THEN
        SELECT solicitud_turno_seq.NEXTVAL
          INTO :NEW.id_solicitud
          FROM DUAL;
    END IF;
END;
/
